'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.createOnSubmitHandler = createOnSubmitHandler;
exports.createOnRemoveHandler = createOnRemoveHandler;
exports.connectSingleItem = connectSingleItem;

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj['default'] = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _uuid = require('uuid');

var _uuid2 = _interopRequireDefault(_uuid);

var _utils = require('../../utils');

var utils = _interopRequireWildcard(_utils);

var _common = require('./common');

/**
 * combines result of two mapStateToProps functions
 *
 * @param fun1
 * @param fun2
 * @returns {object} combined stateProps
 */
function combineMapStateToProps(fun1, fun2) {
  return function (state, ownProps) {
    return Object.assign({}, fun1(state, ownProps), fun2 ? fun2(state, ownProps) : null);
  };
}

var singleObjectMapStateToProps = function singleObjectMapStateToProps(mountPoint, propName) {
  return function singleObjectMapStateToProps(state, ownProps) {
    var id = ownProps.docId;
    return _defineProperty({
      id: id
    }, propName, utils.getObjectFromState(state, mountPoint, id));
  };
};

exports.singleObjectMapStateToProps = singleObjectMapStateToProps;
/**
 * Returns onSubmit handler that dispatch `put` action
 *
 * @param {crud} crud
 * @returns {function}
 */

function createOnSubmitHandler(crud) {
  return function onSubmit(item, data, dispatch) {
    var doc = _extends({}, item, data);
    if (!doc._id) {
      doc._id = crud.mountPoint + '-' + _uuid2['default'].v4();
    }
    dispatch(crud.actions.put(doc));
  };
}

/**
 * createOnRemoveHandler
 *
 * @param crud
 * @returns {function}
 */

function createOnRemoveHandler(crud) {
  return function onRemove(dispatch, items) {
    dispatch(crud.actions.remove(items.toObject()));
  };
}

/**
 * Decorator connects passed component to include single document as `item`.
 *
 * @param {crud} crud
 */

function connectSingleItem(crud, opts, mapStateToProps, mapDispatchToProps) {
  if (opts === undefined) opts = {};

  (0, _invariant2['default'])(typeof crud !== 'undefined', 'crud is required parameter');
  var _opts$propName = opts.propName;
  var propName = _opts$propName === undefined ? 'item' : _opts$propName;

  var loadFunction = function loadFunction(c) {
    c.props.dispatch(crud.actions.get(c.props.id));
  };
  var mergedMapStateToProps = combineMapStateToProps(singleObjectMapStateToProps(crud.mountPoint, propName), mapStateToProps);

  return (0, _common.wrap)(mergedMapStateToProps, mapDispatchToProps, loadFunction, { propName: propName });
}

;